import pydantic
import typing_extensions


class V1AiVoiceGeneratorCreateBodyStyle(typing_extensions.TypedDict):
    """
    The content used to generate speech.
    """

    prompt: typing_extensions.Required[str]
    """
    Text used to generate speech. Starter tier users can use up to 1000 characters, while Creator, Pro, or Business users can use up to 1000.
    """

    voice_name: typing_extensions.Required[
        typing_extensions.Literal[
            "A.R. Rahman",
            "Aamir Khan",
            "Abraham Lincoln",
            "Adam Driver",
            "Ajay Devgn",
            "Akshay Kumar",
            "Alain Delon",
            "Alan Alda",
            "Alan Cumming",
            "Alan Rickman",
            "Albert Einstein",
            "Alex Trebek",
            "Alexz Johnson",
            "Amitabh Bachchan",
            "Amy Poehler",
            "Ana Gasteyer",
            "Andrew Rannells",
            "Ang Lee",
            "Anne Hathaway",
            "Ansel Elgort",
            "Anthony Anderson",
            "Anthony Mackie",
            "Arden Cho",
            "Armie Hammer",
            "Asa Butterfield",
            "B.J. Novak",
            "Barack Obama",
            "Barbara Eden",
            "Bear Grylls",
            "Ben McKenzie",
            "Ben Stiller",
            "Ben Whishaw",
            "Betty White",
            "Bill Nighy",
            "Bill Pullman",
            "Billie Joe Armstrong",
            "Bingbing Li",
            "Blake Shelton",
            "Bob Barker",
            "Bonnie Wright",
            "Booboo Stewart",
            "Brad Paisley",
            "Bradley Steven Perry",
            "Brendan Gleeson",
            "Brian Cox",
            "Bruno Ganz",
            "Bruno Mars",
            "Burt Reynolds",
            "Caitriona Balfe",
            "Caity Lotz",
            "Cameron Boyce",
            "Candice Accola",
            "Carrie Fisher",
            "Carrie Underwood",
            "Casey Affleck",
            "Caterina Scorsone",
            "Cedric the Entertainer",
            "Chace Crawford",
            "Chadwick Boseman",
            "Charles Dance",
            "Charlie Chaplin",
            "Charlie Day",
            "Chiwetel Ejiofor",
            "Chris Hemsworth",
            "Chris Martin",
            "Chris Pine",
            "Christina Hendricks",
            "Christina Ricci",
            "Christopher Mintz-Plasse",
            "Cillian Murphy",
            "Cyndi Lauper",
            "Dakota Fanning",
            "Damian Lewis",
            "Dan Aykroyd",
            "Dan Fogler",
            "Dan Stevens",
            "Daniel Craig",
            "Daniel Dae Kim",
            "Danielle Panabaker",
            "Dave Bautista",
            "David Attenborough",
            "David Letterman",
            "David Oyelowo",
            "David Schwimmer",
            "David Tennant",
            "Denis Leary",
            "Derek Mears",
            "Diane Keaton",
            "Diane Kruger",
            "Dick Van Dyke",
            "Diego Luna",
            "Domhnall Gleeson",
            "Dominic Cooper",
            "Donald Glover",
            "Donald Sutherland",
            "Donald Trump",
            "Donnie Yen",
            "Doutzen Kroes",
            "Dove Cameron",
            "Dr. Dre",
            "Drake Bell",
            "Drew Carey",
            "Eartha Kitt",
            "Eddie Izzard",
            "Edward Asner",
            "Eli Roth",
            "Elisabeth Moss",
            "Elle Fanning",
            "Ellen Burstyn",
            "Elon Musk",
            "Emile Hirsch",
            "Ernie Hudson",
            "Ezra Miller",
            "Felicity Jones",
            "Fergie",
            "Fiona Shaw",
            "Florence Henderson",
            "Forest Whitaker",
            "Francia Raisa",
            "Freddie Highmore",
            "Freida Pinto",
            "Geena Davis",
            "Gemma Arterton",
            "Geri Halliwell",
            "Gillian Jacobs",
            "Gina Carano",
            "Ginnifer Goodwin",
            "Glenn Close",
            "Gloria Steinem",
            "Gordon Ramsay",
            "Greta Gerwig",
            "Gugu Mbatha-Raw",
            "Guy Pearce",
            "Gwendoline Christie",
            "Hailee Steinfeld",
            "Hans Zimmer",
            "Harry Connick Jr.",
            "Harvey Keitel",
            "Helena Bonham Carter",
            "Henry Cavill",
            "Hilary Swank",
            "Howie Mandel",
            "Hugh Bonneville",
            "Hugh Jackman",
            "Hugh Laurie",
            "Idina Menzel",
            "Imelda Staunton",
            "Ingrid Bergman",
            "Irrfan Khan",
            "Isla Fisher",
            "Iwan Rheon",
            "J. K. Simmons",
            "J.J. Abrams",
            "Jack Black",
            "Jack Lemmon",
            "James Earl Jones",
            "Janet Jackson",
            "Jared Leto",
            "Jason Bateman",
            "Jason Segel",
            "Jeff Goldblum",
            "Jennifer Carpenter",
            "Jennifer Coolidge",
            "Jimmy Fallon",
            "Joe Biden",
            "Joe Rogan",
            "John F. Kennedy",
            "Johnny Galecki",
            "Jon Favreau",
            "Joseph Gordon-Levitt",
            "Josh Brolin",
            "Josh Gad",
            "Josh Groban",
            "Julia Louis-Dreyfus",
            "Julia Roberts",
            "Kanye West",
            "Katy Perry",
            "Kesha",
            "Kevin Bacon",
            "Kim Kardashian",
            "Kris Jenner",
            "Kristen Bell",
            "Kristen Stewart",
            "Kristen Wiig",
            "Lorde",
            "Lucille Ball",
            "Marilyn Monroe",
            "Mark Zuckerberg",
            "Matt Smith",
            "Morgan Freeman",
            "Natalie Portman",
            "Prince",
            "Rooney Mara",
            "Samuel L. Jackson",
            "Sean Connery",
            "Stephen Colbert",
            "Steve Carell",
            "Taylor Swift",
            "Tom Hiddleston",
            "Zoe Saldana",
        ]
    ]
    """
    The voice to use for the speech. Available voices: Elon Musk, Mark Zuckerberg, Joe Rogan, Barack Obama, Morgan Freeman, Kanye West, Donald Trump, Joe Biden, Kim Kardashian, Taylor Swift, James Earl Jones, Samuel L. Jackson, Jeff Goldblum, David Attenborough, Sean Connery, Cillian Murphy, Anne Hathaway, Julia Roberts, Natalie Portman, Steve Carell, Amy Poehler, Stephen Colbert, Jimmy Fallon, David Letterman, Alex Trebek, Katy Perry, Prince, Kevin Bacon, Tom Hiddleston, Adam Driver, Alan Rickman, Alexz Johnson, Ana Gasteyer, Andrew Rannells, Arden Cho, Bear Grylls, Ben McKenzie, Ben Stiller, Ben Whishaw, Billie Joe Armstrong, Bingbing Li, Bob Barker, Booboo Stewart, Bradley Steven Perry, Bruno Mars, Caity Lotz, Cameron Boyce, Candice Accola, Carrie Underwood, Casey Affleck, Caterina Scorsone, Cedric the Entertainer, Chace Crawford, Chadwick Boseman, Charlie Day, Chris Hemsworth, Chris Martin, Christopher Mintz-Plasse, Dan Fogler, Dan Stevens, Daniel Dae Kim, Danielle Panabaker, Dave Bautista, David Schwimmer, Denis Leary, Derek Mears, Diego Luna, Donald Glover, Donnie Yen, Doutzen Kroes, Dove Cameron, Dr. Dre, Drake Bell, Elle Fanning, Ernie Hudson, Fergie, Forest Whitaker, Francia Raisa, Freddie Highmore, Gillian Jacobs, Gina Carano, Ginnifer Goodwin, Gordon Ramsay, Guy Pearce, Gwendoline Christie, Hailee Steinfeld, Howie Mandel, Hugh Jackman, Hugh Laurie, J. K. Simmons, Jack Black, Jared Leto, Jennifer Carpenter, Kesha, Kris Jenner, Kristen Bell, Lorde, Matt Smith, Marilyn Monroe, Charlie Chaplin, Albert Einstein, Abraham Lincoln, John F. Kennedy, Lucille Ball, A.R. Rahman, Aamir Khan, Ajay Devgn, Akshay Kumar, Alain Delon, Alan Alda, Alan Cumming, Amitabh Bachchan, Ang Lee, Ansel Elgort, Anthony Anderson, Anthony Mackie, Armie Hammer, Asa Butterfield, B.J. Novak, Barbara Eden, Betty White, Bill Nighy, Bill Pullman, Blake Shelton, Bonnie Wright, Brad Paisley, Brendan Gleeson, Brian Cox, Bruno Ganz, Burt Reynolds, Carrie Fisher, Charles Dance, Chiwetel Ejiofor, Chris Pine, Christina Hendricks, Christina Ricci, Cyndi Lauper, Dakota Fanning, Damian Lewis, Dan Aykroyd, Daniel Craig, David Oyelowo, David Tennant, Diane Keaton, Diane Kruger, Dick Van Dyke, Domhnall Gleeson, Dominic Cooper, Donald Sutherland, Drew Carey, Eartha Kitt, Eddie Izzard, Edward Asner, Eli Roth, Elisabeth Moss, Ellen Burstyn, Emile Hirsch, Ezra Miller, Felicity Jones, Fiona Shaw, Florence Henderson, Freida Pinto, Geena Davis, Gemma Arterton, Geri Halliwell, Glenn Close, Gloria Steinem, Greta Gerwig, Gugu Mbatha-Raw, Hans Zimmer, Harry Connick Jr., Harvey Keitel, Helena Bonham Carter, Henry Cavill, Hilary Swank, Hugh Bonneville, Idina Menzel, Imelda Staunton, Ingrid Bergman, Irrfan Khan, Isla Fisher, Iwan Rheon, Jack Lemmon, Janet Jackson, Jason Bateman, Jason Segel, Jennifer Coolidge, Johnny Galecki, Jon Favreau, Joseph Gordon-Levitt, Josh Brolin, Josh Gad, Josh Groban, Julia Louis-Dreyfus, Kristen Stewart, Kristen Wiig, Rooney Mara, Caitriona Balfe, J.J. Abrams, Zoe Saldana
    """


class _SerializerV1AiVoiceGeneratorCreateBodyStyle(pydantic.BaseModel):
    """
    Serializer for V1AiVoiceGeneratorCreateBodyStyle handling case conversions
    and file omissions as dictated by the API
    """

    model_config = pydantic.ConfigDict(
        populate_by_name=True,
    )

    prompt: str = pydantic.Field(
        alias="prompt",
    )
    voice_name: typing_extensions.Literal[
        "A.R. Rahman",
        "Aamir Khan",
        "Abraham Lincoln",
        "Adam Driver",
        "Ajay Devgn",
        "Akshay Kumar",
        "Alain Delon",
        "Alan Alda",
        "Alan Cumming",
        "Alan Rickman",
        "Albert Einstein",
        "Alex Trebek",
        "Alexz Johnson",
        "Amitabh Bachchan",
        "Amy Poehler",
        "Ana Gasteyer",
        "Andrew Rannells",
        "Ang Lee",
        "Anne Hathaway",
        "Ansel Elgort",
        "Anthony Anderson",
        "Anthony Mackie",
        "Arden Cho",
        "Armie Hammer",
        "Asa Butterfield",
        "B.J. Novak",
        "Barack Obama",
        "Barbara Eden",
        "Bear Grylls",
        "Ben McKenzie",
        "Ben Stiller",
        "Ben Whishaw",
        "Betty White",
        "Bill Nighy",
        "Bill Pullman",
        "Billie Joe Armstrong",
        "Bingbing Li",
        "Blake Shelton",
        "Bob Barker",
        "Bonnie Wright",
        "Booboo Stewart",
        "Brad Paisley",
        "Bradley Steven Perry",
        "Brendan Gleeson",
        "Brian Cox",
        "Bruno Ganz",
        "Bruno Mars",
        "Burt Reynolds",
        "Caitriona Balfe",
        "Caity Lotz",
        "Cameron Boyce",
        "Candice Accola",
        "Carrie Fisher",
        "Carrie Underwood",
        "Casey Affleck",
        "Caterina Scorsone",
        "Cedric the Entertainer",
        "Chace Crawford",
        "Chadwick Boseman",
        "Charles Dance",
        "Charlie Chaplin",
        "Charlie Day",
        "Chiwetel Ejiofor",
        "Chris Hemsworth",
        "Chris Martin",
        "Chris Pine",
        "Christina Hendricks",
        "Christina Ricci",
        "Christopher Mintz-Plasse",
        "Cillian Murphy",
        "Cyndi Lauper",
        "Dakota Fanning",
        "Damian Lewis",
        "Dan Aykroyd",
        "Dan Fogler",
        "Dan Stevens",
        "Daniel Craig",
        "Daniel Dae Kim",
        "Danielle Panabaker",
        "Dave Bautista",
        "David Attenborough",
        "David Letterman",
        "David Oyelowo",
        "David Schwimmer",
        "David Tennant",
        "Denis Leary",
        "Derek Mears",
        "Diane Keaton",
        "Diane Kruger",
        "Dick Van Dyke",
        "Diego Luna",
        "Domhnall Gleeson",
        "Dominic Cooper",
        "Donald Glover",
        "Donald Sutherland",
        "Donald Trump",
        "Donnie Yen",
        "Doutzen Kroes",
        "Dove Cameron",
        "Dr. Dre",
        "Drake Bell",
        "Drew Carey",
        "Eartha Kitt",
        "Eddie Izzard",
        "Edward Asner",
        "Eli Roth",
        "Elisabeth Moss",
        "Elle Fanning",
        "Ellen Burstyn",
        "Elon Musk",
        "Emile Hirsch",
        "Ernie Hudson",
        "Ezra Miller",
        "Felicity Jones",
        "Fergie",
        "Fiona Shaw",
        "Florence Henderson",
        "Forest Whitaker",
        "Francia Raisa",
        "Freddie Highmore",
        "Freida Pinto",
        "Geena Davis",
        "Gemma Arterton",
        "Geri Halliwell",
        "Gillian Jacobs",
        "Gina Carano",
        "Ginnifer Goodwin",
        "Glenn Close",
        "Gloria Steinem",
        "Gordon Ramsay",
        "Greta Gerwig",
        "Gugu Mbatha-Raw",
        "Guy Pearce",
        "Gwendoline Christie",
        "Hailee Steinfeld",
        "Hans Zimmer",
        "Harry Connick Jr.",
        "Harvey Keitel",
        "Helena Bonham Carter",
        "Henry Cavill",
        "Hilary Swank",
        "Howie Mandel",
        "Hugh Bonneville",
        "Hugh Jackman",
        "Hugh Laurie",
        "Idina Menzel",
        "Imelda Staunton",
        "Ingrid Bergman",
        "Irrfan Khan",
        "Isla Fisher",
        "Iwan Rheon",
        "J. K. Simmons",
        "J.J. Abrams",
        "Jack Black",
        "Jack Lemmon",
        "James Earl Jones",
        "Janet Jackson",
        "Jared Leto",
        "Jason Bateman",
        "Jason Segel",
        "Jeff Goldblum",
        "Jennifer Carpenter",
        "Jennifer Coolidge",
        "Jimmy Fallon",
        "Joe Biden",
        "Joe Rogan",
        "John F. Kennedy",
        "Johnny Galecki",
        "Jon Favreau",
        "Joseph Gordon-Levitt",
        "Josh Brolin",
        "Josh Gad",
        "Josh Groban",
        "Julia Louis-Dreyfus",
        "Julia Roberts",
        "Kanye West",
        "Katy Perry",
        "Kesha",
        "Kevin Bacon",
        "Kim Kardashian",
        "Kris Jenner",
        "Kristen Bell",
        "Kristen Stewart",
        "Kristen Wiig",
        "Lorde",
        "Lucille Ball",
        "Marilyn Monroe",
        "Mark Zuckerberg",
        "Matt Smith",
        "Morgan Freeman",
        "Natalie Portman",
        "Prince",
        "Rooney Mara",
        "Samuel L. Jackson",
        "Sean Connery",
        "Stephen Colbert",
        "Steve Carell",
        "Taylor Swift",
        "Tom Hiddleston",
        "Zoe Saldana",
    ] = pydantic.Field(
        alias="voice_name",
    )
